/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gwibber-application.h"
#include <QtQuick/QQuickView>
#include <QtGui/QGuiApplication>
#include <QtQml/QQmlEngine>
#include <QtQml>
#include <QtQuick/QQuickItem>
#include <QtCore/QCoreApplication>
#include <dee-1.0/dee.h>
#include <deelistmodel.h>


static void printUsage(const QStringList& arguments)
{
    qDebug() << "usage:"
             << arguments.at(0).toUtf8().constData()
             << "[-testability] [--fullscreen]";
}

GwibberApplication::GwibberApplication(int &argc, char **argv)
    : QGuiApplication(argc, argv), m_view(0)
{
    // The testability driver is only loaded by QApplication but not by QGuiApplication.
    // However, QApplication depends on QWidget which would add some unneeded overhead => Let's load the testability driver on our own.
    if (arguments().contains(QLatin1String("-testability"))) {
        QLibrary testLib(QLatin1String("qttestability"));
        if (testLib.load()) {
            typedef void (*TasInitialize)(void);
            TasInitialize initFunction = (TasInitialize)testLib.resolve("qt_testability_init");
            if (initFunction) {
                initFunction();
            } else {
                qCritical("Library qttestability resolve failed!");
            }
        } else {
            qCritical("Library qttestability load failed!");
        }
    }
}

inline bool isRunningInstalled() {
    static bool installed = (QCoreApplication::applicationDirPath() ==
                             QDir(("/usr/bin")).canonicalPath());
    return installed;
}

inline QString getAppDirectory() {
    if (isRunningInstalled()) {
        return QString("/usr/share/gwibber-qml/qml/");
    } else {
        return QString(QCoreApplication::applicationDirPath() + "/../qml/");
    }
}

void
onSynchronizedChanged(GObject* parent, GParamSpec *pspec, gpointer user_data)
{
    DeeListModel *results_model = (DeeListModel*)user_data;
    DeeModel *model = (DeeModel*)parent;
    DeeFilter _sort_filter;
    dee_filter_new_collator (6, &_sort_filter);
    DeeModel* _sorted_model = dee_filter_model_new (model, &_sort_filter);
    results_model->setModel (_sorted_model);
}


bool GwibberApplication::setup()
{
    GwibberApplication::setApplicationName("Gwibber");
    QQuickView* m_view = new QQuickView();
    QQmlContext* context = m_view->rootContext();
    DeeModel* _shared_model = dee_shared_model_new ("com.canonical.Friends.Streams");
    DeeListModel *_m = new DeeListModel ();
    context->setContextProperty("_friendsModel", _m);
    g_signal_connect(_shared_model, "notify::synchronized", G_CALLBACK(onSynchronizedChanged), G_OBJECT(_m));

    m_view->setResizeMode(QQuickView::SizeRootObjectToView);
    m_view->setTitle("Gwibber");
    m_view->engine()->setBaseUrl(QUrl::fromLocalFile(getAppDirectory()));
    QObject::connect(m_view->engine(), SIGNAL(quit()), qApp, SLOT(quit()));
    m_view->setSource(QUrl::fromLocalFile("GwibberApplication.qml"));
    m_view->setColor("transparent");
    connect(m_view->engine(), SIGNAL(quit()), SLOT(quit()));
    if (arguments().contains(QLatin1String("--fullscreen"))) m_view->showFullScreen();
    else m_view->show();

    return true;
}

GwibberApplication::~GwibberApplication()
{
    if (m_view) {
        delete m_view;
    }
}
